//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//
//=============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Diagnostics;

using YAC_NET.Data;
using YAC_NET.Properties;

namespace YAC_NET.Controls
{
    /// <summary>
    /// 
    /// </summary>
    public partial class ListenerControl : UserControl
    {
        public ListenerControl()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Delete the current item
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            listenersBindingSource.RemoveCurrent();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listenersDataGridView_CellValueChanged(object sender, DataGridViewCellEventArgs e)
        {

            if (e.RowIndex == -1) return;
            if (e.ColumnIndex == -1) return;

            listenersDataGridView.Rows[e.RowIndex].Cells[e.ColumnIndex].ErrorText = string.Empty;

            string name = listenersDataGridView.Rows[e.RowIndex].Cells[e.ColumnIndex].Value as string;

            if (string.IsNullOrEmpty(name)) return;

            try
            {
                this.Cursor = Cursors.WaitCursor;

                IPHostEntry entry = Dns.GetHostEntry(name);

                // column "1" is the address column
                listenersDataGridView.Rows[e.RowIndex].Cells[1].Value = entry.AddressList[0].ToString();

            }
            catch (Exception ex)
            {
                Trace.TraceError("Error trying to resolve IP address from name: {0}\r\n{1}", name, ex.ToString());
                listenersDataGridView.Rows[e.RowIndex].Cells[e.ColumnIndex].ErrorText = Resources.Message_NoSuchHost;
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }
        }

        /// <summary>
        /// Select the row on right click
        /// Has to be on the MouseDown event and not the CellMouseDown event!
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listenersDataGridView_MouseDown(object sender, MouseEventArgs e)
        {
            Utility.Utility.SelectDataGridViewRowOnRightClick(sender, e);

        }

        /// <summary>
        /// If there are no rows selected, don't pop up the context menu
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myContextMenuStrip_Opening(object sender, CancelEventArgs e)
        {
            if (listenersDataGridView.SelectedCells.Count == 0)
                e.Cancel = true;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ListenerControl_Load(object sender, EventArgs e)
        {
            if (this.DesignMode)
                return;

            listenersBindingSource.DataSource = Configuration.Current.Listeners;
        }

    }
}
