//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//=============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

using YAC_NET.Data;
using YAC_NET.Properties;

namespace YAC_NET.Controls
{
    /// <summary>
    /// 
    /// </summary>
    public partial class PhoneBookControl : UserControl
    {
        public PhoneBookControl()
        {
            InitializeComponent();
        }

        bool m_selectLast = false;

        /// <summary>
        /// 
        /// </summary>
        public void SelectNewest()
        {
            m_selectLast = true;
        }

        /// <summary>
        /// Load the forms and initialize
        /// if SelectNewest was called, select the last item in the list 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PhoneBookControl_Load(object sender, EventArgs e)
        {
            if (this.DesignMode)
                return;
            
            myTableBindingSource.DataSource = Configuration.Current.NumberTranslation.Table;

            if (m_selectLast)
            {
                //myTableBindingSource.MoveLast();

                // select the last item in the table
                myTableDataGridView.Rows[myTableDataGridView.Rows.Count-1].Cells[0].Selected = true;
            }

        }

        /// <summary>
        /// Delete the currently selected item
        /// Since this control allows you to select cells
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            foreach (DataGridViewRow row in myTableDataGridView.SelectedRows)
                myTableDataGridView.Rows.Remove(row);
            foreach (DataGridViewCell cell in myTableDataGridView.SelectedCells)
                myTableDataGridView.Rows.RemoveAt(cell.RowIndex);
            //myTableBindingSource.RemoveCurrent();
        }

        /// <summary>
        /// Clear all of the numbers from the phone book
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void clearToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(
                this,
                Resources.WarningDeletingPhoneBook,
                Resources.Caption_Warning,
                MessageBoxButtons.YesNo,
                 MessageBoxIcon.Warning) == DialogResult.No)
                return;

            myTableBindingSource.Clear();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tableDataGridView_MouseDown(object sender, MouseEventArgs e)
        {
            Utility.Utility.SelectDataGridViewRowOnRightClick(sender, e);
        }

        /// <summary>
        /// Called when a cell is about to be formatted for display
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myTableDataGridView_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e)
        {
            // if it is the number colunm
            if (e.ColumnIndex == 0)
            {
                try
                {
                    e.Value = Configuration.Current.Format.FormatNumber(e.Value as string);
                }
                catch (Exception ex)
                {
                    Trace.TraceError("Error formatting a number for the phone book",ex.ToString());
                    e.FormattingApplied = false;
                }
                finally
                {
                    e.FormattingApplied = true;
                }
            }
        }

        /// <summary>
        /// Called when a typed entry is about to pushed to the backing data structure
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myTableDataGridView_CellParsing(object sender, DataGridViewCellParsingEventArgs e)
        {
            // if it is the number column remove all formatting characters and spaces
            if (e.ColumnIndex == 0)
            {
                try
                {
                    e.Value = Configuration.Current.Format.ParseNumber(e.Value as string);
                }
                catch (Exception ex)
                {
                    Trace.TraceError("Error parsing a number for the phone book", ex.ToString());
                }
            }
        }

        /// <summary>
        /// Handle an error entering data into a cell
        /// also handles formatting or parsing errors
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myTableDataGridView_DataError(object sender, DataGridViewDataErrorEventArgs e)
        {
            Trace.TraceError("Error with data in a cell ({0},{1})\r\n{2}", 
                e.ColumnIndex, 
                e.RowIndex, 
                e.Exception.ToString());
        }

        /// <summary>
        /// If the number of selected cells is zero, cancel this menu
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myContextMenuStrip_Opening(object sender, CancelEventArgs e)
        {
            if (myTableDataGridView.SelectedCells.Count == 0)
                e.Cancel = true;
        }

    }
}
