//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//=============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Collections.Specialized;

using YAC_NET.Data;
using YAC_NET.Utility;
using YAC_NET.Properties;
using System.Diagnostics;

namespace YAC_NET.Forms
{
    /// <summary>
    /// The call log forms displays the call log, 
    /// and allows the user to clear or remove items from the log, 
    /// and to print the log
    /// </summary>
    public partial class CallLogForm : Form
    {
        public CallLogForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Show this form
        /// </summary>
        /// <param name="mainForm"></param>
        new internal static void Show(IWin32Window mainForm)
        {
            FormUtil.Show<CallLogForm>(mainForm);
        }

        #region fields
        CallLog m_callLog;
        string m_currentFilePath;
        #endregion

        /// <summary>
        /// Load the call log into the form
        /// </summary>
        private void LoadCallLog()
        {
            if (m_callLog == null)
            {
                MessageBox.Show(
                    this,
                    Resources.Error_LoadingCallLog,
                    Resources.Caption_Error,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
            else
            {

                // set the data sources
                // you can edit the call log
                callerIDTypeBindingSource.DataSource = m_callLog.CallerIDType;

                // sort the calls by date descending (newest to oldest)
                m_callLog.Sort(SortOrder.Descending);

                callerIDTypeDataGridView.Columns[0].HeaderCell.SortGlyphDirection = SortOrder.Descending;

                myDateToolStripLabel.Text = string.Format("{0}/{1}", m_callLog.Month, m_callLog.Year);

                Invalidate();

            }
        }

        /// <summary>
        /// Load the call log from the XML file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CallLogForm_Load(object sender, EventArgs e)
        {
            // read the log
            m_callLog = CallLogger.ReadLog();

            LoadCallLog();

            // you can also set the option to log or not here
            configurationBindingSource.DataSource = Configuration.Current;

            // bind a collection of call logs
            logFileBindingSource.DataSource = CallLog.GetCallLogs();
        }

        ///// <summary>
        ///// Clear the log
        ///// </summary>
        ///// <param name="sender"></param>
        ///// <param name="e"></param>
        //private void myClearButton_Click(object sender, EventArgs e)
        //{
        //    if (MessageBox.Show(
        //        this,
        //        Resources.WarningDeletingCallLog,
        //        Resources.Warning,
        //        MessageBoxButtons.YesNo,
        //         MessageBoxIcon.Warning) == DialogResult.No)
        //        return;

        //    callerIDTypeBindingSource.Clear();
        //}

        /// <summary>
        /// Add the currently selected item to the phone book.
        /// Select the current item, put it in the phone book,
        /// and then open the phone book to show the user the new entry
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void addToPhoneBookToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CallerIDType callerID = callerIDTypeBindingSource.Current as CallerIDType;
            if (callerID == null)
                return;
            if (!Configuration.Current.NumberTranslation.Add(callerID))
            {
                MessageBox.Show(
                    this,
                    Resources.Error_AddingPhoneBook, 
                    Resources.Caption_Error,
                    MessageBoxButtons.OK, 
                    MessageBoxIcon.Error);
            }

            PhoneBookForm form = new PhoneBookForm();
            form.SelectNewest();
            form.ShowDialog(this.Parent);

        }

        ///// <summary>
        ///// Delete the currently selected item
        ///// </summary>
        ///// <param name="sender"></param>
        ///// <param name="e"></param>
        //private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        //{
        //    callerIDTypeBindingSource.RemoveCurrent();
        //}

        /// <summary>
        /// When you close the log, save it to persist any edits
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myOKButton_Click(object sender, EventArgs e)
        {
            //CallLogger.SaveLog(m_callLog);
            Close();
        }

        /// <summary>
        /// Print the current call log
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myPrintButton_Click(object sender, EventArgs e)
        {
            if (myPrintDialog.ShowDialog(this) != DialogResult.OK)
                return;
            myPrintDocument.PrinterSettings = myPrintDialog.PrinterSettings;
            myPrintDocument.Print(); 
        }

        /// <summary>
        /// Formula to select a DatagridView row on right click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void callerIDTypeDataGridView_MouseDown(object sender, MouseEventArgs e)
        {
            Utility.Utility.SelectDataGridViewRowOnRightClick(sender, e);
        }

        /// <summary>
        /// Print a page of the output
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myPrintDocument_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
        {
            // create a bitmap the size of the form
            Bitmap bmp = new Bitmap(callerIDTypeDataGridView.Width, callerIDTypeDataGridView.Height);

            // draw the form image to the bitmap
            callerIDTypeDataGridView.DrawToBitmap(bmp, new Rectangle(0, 0, callerIDTypeDataGridView.Width, callerIDTypeDataGridView.Height));

            // draw the bitmap image of the form onto the graphics surface
            e.Graphics.DrawImage(bmp, new Point(0, 0));

        }

        /// <summary>
        /// When the user clicks on a column header
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void callerIDTypeDataGridView_ColumnHeaderMouseClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            if (e.ColumnIndex == 0)
            {
                if (callerIDTypeDataGridView.Columns[0].HeaderCell.SortGlyphDirection == SortOrder.Ascending)
                {
                    // sort the calls by date descending (newest to oldest)
                    //callerIDTypeDataGridView.Sort(new DateRowComparer(SortOrder.Descending));
                    m_callLog.CallerIDType.Sort(new CallerIDTypeComparer(SortOrder.Descending));
                    callerIDTypeDataGridView.Columns[0].HeaderCell.SortGlyphDirection = SortOrder.Descending;
                }
                else
                {
                    // sort the calls by date descending (newest to oldest)
                    m_callLog.CallerIDType.Sort(new CallerIDTypeComparer(SortOrder.Ascending));
                    callerIDTypeDataGridView.Columns[0].HeaderCell.SortGlyphDirection = SortOrder.Ascending;
                }
            }
        }

        /// <summary>
        /// Called when the current log file changes
        /// Load the new log file and update the databinding
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void logFileBindingSource_CurrentChanged(object sender, EventArgs e)
        {
            string fullFilePath = (string)logFileBindingSource.Current;
            if (m_currentFilePath == fullFilePath)
                return;
            m_currentFilePath = fullFilePath;
            
            string fileName = Path.GetFileName(fullFilePath);
            myLogFileNameToolStripTextBox.Text = fileName;

            m_callLog = (m_currentFilePath == CallLog.FileName) ? 
                CallLog.Load() :
                CallLog.Load(fullFilePath);

            LoadCallLog();

        }

        /// <summary>
        /// Delete the currently selected log file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bindingNavigatorDeleteItem_Click(object sender, EventArgs e)
        {
            string fullFilePath = (string)logFileBindingSource.Current;
            if (MessageBox.Show(
                this,
                Resources.WarningDeletingCallLog,
                Resources.Caption_Warning,
                MessageBoxButtons.YesNo,
                 MessageBoxIcon.Warning) == DialogResult.No)
                return;

            try
            {
                logFileBindingSource.RemoveCurrent();

                File.Delete(fullFilePath);
            }
            catch (Exception ex)
            {
                string message = string.Format("Error deleting {0}:\r\n{1}", Path.GetFileName(fullFilePath), ex.ToString());
                Trace.TraceError(message);

                MessageBox.Show(
                    this,
                    Resources.Caption_Error,
                    message,
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void myExportButton_Click(object sender, EventArgs e)
        {
            ExportForm.Show(this);
        }

    }

}