//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//=============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using YAC_NET.Properties;
using YAC_NET.Data;
using YAC_NET.Utility;

namespace YAC_NET.Forms
{
    public partial class WelcomeForm : Form
    {
        public WelcomeForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Show the dialog box
        /// </summary>
        /// <param name="mainForm"></param>
        new internal static void Show(IWin32Window mainForm)
        {
            FormUtil.Show<WelcomeForm>(mainForm);
        }

        int m_tabPageIndex = 0;

        /// <summary>
        /// Load the form and initialize the buttons
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void WelcomeForm_Load(object sender, EventArgs e)
        {
            myDoneButton.Enabled = false;
            myTabControl.ItemSize = new Size(1, 1);

            // set up the "install service" button
            if (Utility.Utility.ServiceExists(Resources.ServiceName))
            {
                myServiceLabel.Text = string.Format(Resources.Message_Service);
                myRunAsServiceCheckBox.Checked = true;
            }
            else
            {
                myServiceLabel.Text = string.Format(Resources.Message_DesktopProgram);
            }

        }

        /// <summary>
        /// The next button moves to the next tab unless you're on the last tab
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myNextButton_Click(object sender, EventArgs e)
        {
            m_tabPageIndex++;
            myTabControl.SelectTab(m_tabPageIndex);

            myPreviousButton.Enabled = true;

            if (m_tabPageIndex == myTabControl.TabPages.Count - 1)
            {
                myDoneButton.Enabled = true;
                myNextButton.Enabled = false;
                SetDoneLabel();
            }
            else
            {
                myNextButton.Enabled = true;
            }

        }

        /// <summary>
        /// the previous button moves to the last tab unless you're on the first tab
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myPreviousButton_Click(object sender, EventArgs e)
        {
            m_tabPageIndex--;
            myTabControl.SelectTab(m_tabPageIndex);

            myDoneButton.Enabled = true;
            myNextButton.Enabled = true;

            if (m_tabPageIndex == 0)
            {
                myPreviousButton.Enabled = false;
            }
            else
            {
                myPreviousButton.Enabled = true;
            }

        }

        /// <summary>
        /// Install the service (run YAC as a service)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void myRunAsServiceCheckBox_Click(object sender, EventArgs e)
        {
            try
            {
                this.Cursor = Cursors.WaitCursor;
                if (Utility.Utility.ServiceExists(Resources.ServiceName))
                {
                    myServiceLabel.Text = string.Format(Resources.Message_DesktopProgram);
                    myDoneConfigureLabel.Text = string.Format(Resources.Message_YACReadyProgram, Configuration.Current.Modem.Port);
                    Program.DoUninstall();
                }
                else
                {
                    myServiceLabel.Text = string.Format(Resources.Message_Service);
                    myDoneConfigureLabel.Text = string.Format(Resources.Message_YACReadyService, Configuration.Current.Modem.Port);
                    Program.DoInstall();
                }
            }
            catch (Exception ex)
            {
                ExceptionManager.CriticalException(ex);
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }

        }

        /// <summary>
        /// Set the final panel of the setup wizard
        /// </summary>
        private void SetDoneLabel()
        {
            if (Utility.Utility.ServiceExists(Resources.ServiceName))
                myDoneConfigureLabel.Text = 
                    string.Format(
                        Resources.Message_YACReadyService, 
                        Configuration.Current.Modem.Port);
            else
                myDoneConfigureLabel.Text = 
                    string.Format(
                        Resources.Message_YACReadyProgram, 
                        Configuration.Current.Modem.Port);

        }

        private void myCallerIDDebugButton_Click(object sender, EventArgs e)
        {
            Configuration.Current.Notify(CallerIDType.CreateDebugCallerID());
        }

    }
}