//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//=============================================================================
using System;
using System.Collections.Generic;
using System.Text;

namespace YAC_NET
{
    /// <summary>
    /// EnumValue represents an enumeration value and it's name
    /// </summary>
    public class EnumValue
    {
        public EnumValue(string name, object value)
        {
            m_name = name;
            m_value = value;
        }

        private string m_name;
        private object m_value;

        /// <summary>
        /// Name of the enum value
        /// </summary>
        public string Name
        {
            get { return m_name; }
            set { m_name = value; }
        }

        /// <summary>
        /// Value of the enumeration
        /// </summary>
        public object Value
        {
            get { return m_value; }
            set { m_value = value; }
        }

    }

    /// <summary>
    /// A collection of enumValues.
    /// Used to show enumeration values in a combobox.
    /// Set this as the datasource of the combobox, 
    /// then set the displaymember to "Name", value member to "Value"
    /// </summary>
    public class EnumCollection : List<EnumValue>
    {
        private Type m_type;

        /// <summary>
        /// Constructor, takes an enumeration type
        /// </summary>
        /// <param name="enumeration"></param>
        public EnumCollection(Enum enumeration)
        {
            Init(enumeration.GetType());
        }

        /// <summary>
        /// Constructor, takes a type which is a enumeration
        /// </summary>
        /// <param name="enumType"></param>
        public EnumCollection(Type enumType)
        {
            Init(enumType);
        }

        /// <summary>
        /// Initialize this class
        /// </summary>
        /// <param name="enumType"></param>
        public void Init(Type enumType)
        {
            m_type = enumType;

            foreach (string name in Enum.GetNames(m_type))
            {
                object val = Enum.Parse(m_type, name);
                this.Add(new EnumValue(name, val));
            }
        }

    }
}
