//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//=============================================================================
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.ServiceProcess;

using YAC_NET.Properties;
using YAC_NET.Data;

namespace YAC_NET
{
    /// <summary>
    /// Implements the YAC service
    /// </summary>
    partial class YacService : ServiceBase
    {
        /// <summary>
        /// Constructor, 
        /// create the IYAC interface, 
        /// handle the callerID event, 
        /// </summary>
        public YacService()
        {
            m_yac = new YAC();

            // handle the callerID and phone ring events
            m_yac.CallerID += new CallerIDDelegate(m_yac_CallerID);
            m_yac.PhoneRing += new PhoneRingDelegate(m_yac_PhoneRing);

            InitializeComponent();
            this.ServiceName = Resources.ServiceName;
        }

        /// <summary>
        /// Handle the phone ring event
        /// notify the listeners and let them handle it
        /// </summary>
        void m_yac_PhoneRing()
        {
            Configuration.Current.PhoneRing();
        }

        /// <summary>
        /// Handle a caller ID event
        /// </summary>
        /// <param name="callerID"></param>
        void m_yac_CallerID(CallerIDType callerID)
        {
            // look up the number in the phone book to give it a friendly name
            Configuration.Current.NumberTranslation.Translate(callerID);

            // notify Listeners
            Configuration.Current.Notify(callerID);

            // if we're logging calls, send them to the log
            if (Configuration.Current.LogCalls)
                CallLogger.LogCall(callerID);
        }

        YAC m_yac;

        /// <summary>
        /// Start the service
        /// </summary>
        /// <param name="args"></param>
        protected override void OnStart(string[] args)
        {
            m_yac.Start();
        }

        /// <summary>
        /// Stop the service
        /// </summary>
        protected override void OnStop()
        {
            m_yac.Stop();
        }
    }
}
