//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright   2007 Michael F Coyle
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//=============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Reflection;
using System.IO;
using System.Diagnostics;

using YAC_NET.Properties;
using YAC_NET.Utility;

namespace YAC_NET.Forms
{
    /// <summary>
    /// The About box is supposed to show imporant information about the 
    /// software.
    /// </summary>
    partial class AboutBox : Form
    {

        /// <summary>
        /// Constructor
        /// </summary>
        public AboutBox()
        {
            InitializeComponent();
        }

        #region Assembly Attribute Accessors

        public string AssemblyTitle
        {
            get
            {
                // Get all Title attributes on this assembly
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyTitleAttribute), false);
                // If there is at least one Title attribute
                if (attributes.Length > 0)
                {
                    // Select the first one
                    AssemblyTitleAttribute titleAttribute = (AssemblyTitleAttribute)attributes[0];
                    // If it is not an empty string, return it
                    if (titleAttribute.Title != "")
                        return titleAttribute.Title;
                }
                // If there was no Title attribute, or if the Title attribute was the empty string, return the .exe name
                return System.IO.Path.GetFileNameWithoutExtension(Assembly.GetExecutingAssembly().CodeBase);
            }
        }

        public string AssemblyVersion
        {
            get
            {
                return Assembly.GetExecutingAssembly().GetName().Version.ToString();
            }
        }

        public string AssemblyDescription
        {
            get
            {
                // Get all Description attributes on this assembly
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyDescriptionAttribute), false);
                // If there aren't any Description attributes, return an empty string
                if (attributes.Length == 0)
                    return "";
                // If there is a Description attribute, return its value
                return ((AssemblyDescriptionAttribute)attributes[0]).Description;
            }
        }

        public string AssemblyProduct
        {
            get
            {
                // Get all Product attributes on this assembly
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyProductAttribute), false);
                // If there aren't any Product attributes, return an empty string
                if (attributes.Length == 0)
                    return "";
                // If there is a Product attribute, return its value
                return ((AssemblyProductAttribute)attributes[0]).Product;
            }
        }

        public string AssemblyCopyright
        {
            get
            {
                // Get all Copyright attributes on this assembly
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyCopyrightAttribute), false);
                // If there aren't any Copyright attributes, return an empty string
                if (attributes.Length == 0)
                    return "";
                // If there is a Copyright attribute, return its value
                return ((AssemblyCopyrightAttribute)attributes[0]).Copyright;
            }
        }

        public string AssemblyCompany
        {
            get
            {
                // Get all Company attributes on this assembly
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyCompanyAttribute), false);
                // If there aren't any Company attributes, return an empty string
                if (attributes.Length == 0)
                    return "";
                // If there is a Company attribute, return its value
                return ((AssemblyCompanyAttribute)attributes[0]).Company;
            }
        }
        #endregion

        /// <summary>
        /// Static show message
        /// </summary>
        /// <param name="mainForm"></param>
        new internal static void Show(IWin32Window mainForm)
        {
            FormUtil.Show<AboutBox>(mainForm);
        }

        /// <summary>
        /// Load and initialize the about box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AboutBox_Load(object sender, EventArgs e)
        {
            this.Text = string.Format("About {0}", Application.ProductName);
            myAppTitleLabel.Text = string.Format("{0} {1}",
                Application.ProductName,
                Application.ProductVersion);
            myAppCopyrightLabel.Text = string.Format("{0} {1} ",
                Application.ProductName,
                Application.ProductVersion);
            myAppDescriptionLabel.Text = AssemblyDescription;
        }

        /// <summary>
        /// open a clicked link (control has url in the tag)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void link_Click(object sender, EventArgs e)
        {
            Control control = sender as Control;
            if (control == null) return;
            string command = control.Tag as string;
            if (string.IsNullOrEmpty(command)) return;
            Process.Start(command);
        }
    }
}
