using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using YAC_NET.Data;
using System.IO;
using System.Diagnostics;

namespace YAC_NET.Forms
{
    public partial class ExportForm : Form
    {
        public ExportForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Show this form
        /// </summary>
        /// <param name="mainForm"></param>
        new internal static void Show(IWin32Window mainForm)
        {
            ExportForm box = new ExportForm();
            box.ShowDialog(mainForm);
        }

        private void myOKButton_Click(object sender, EventArgs e)
        {
            if (myCSVSaveFileDialog.ShowDialog(this) != DialogResult.OK)
                return;

            try
            {
                this.Cursor = Cursors.WaitCursor;

                // Create a call log object
                CallLog callLog = CallLog.Load();
                callLog.Sort(SortOrder.Descending);

                if (this.myExportAllLogFilesCheckBox.Checked)
                {
                    foreach (string file in CallLog.GetCallLogs())
                    {
                        if (file == CallLog.FileName)
                            continue;
                        CallLog oldLog = CallLog.Load(file);
                        oldLog.Sort(SortOrder.Descending);
                        callLog.CallerIDType.AddRange(oldLog.CallerIDType);
                    }
                }

                StringWriter sw = new StringWriter();

                if (myExportColumnHeadersCheckBox.Checked)
                {
                    if (mySeparateDateTimeCheckBox.Checked)
                        sw.WriteLine("{0}, {1}, {2}, {3}, {4}", "#", "Name", "Number", "Date", "Time");
                    else
                        sw.WriteLine("{0}, {1}, {2}, {3}", "#", "Name", "Number", "Date");
                }

                int i = 0;
                foreach (CallerIDType callerId in callLog.CallerIDType)
                {
                    if (mySeparateDateTimeCheckBox.Checked)
                        sw.WriteLine("{0}, {1}, {2}, {3:d}, {4:T}", i++, callerId.Name, callerId.Number, callerId.Date, callerId.Date);
                    else
                        sw.WriteLine("{0}, {1}, {2}, {3}", i++, callerId.Name, callerId.Number, callerId.Date);
                }

                using (StreamWriter writer = new StreamWriter(myCSVSaveFileDialog.FileName))
                    writer.Write(sw.ToString());
            }
            catch (Exception ex)
            {
                string message = string.Format("Error saving XSV file {0}:\r\n{1}", myCSVSaveFileDialog.FileName, ex.ToString());
                Trace.TraceError(message);

            }
            finally
            {
                this.Cursor = Cursors.Default;
            }

            Close();
        }
    }
}