//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//=============================================================================
using System;
using System.Collections;
using System.ComponentModel;
using System.Configuration.Install;
using System.Configuration;
using System.ServiceProcess;
using System.Diagnostics;
using System.Reflection;

using YAC_NET.Properties;

// See below for how to prompt the user for the account to use to execute this service
//using System.ServiceProcess.Design; 
// for ServiceInstallerDialog 
//using System.Windows.Forms;

namespace YAC_NET.Install
{

    /// <summary>
    /// 
    /// This class installs or uninstalls this service.
    /// This class allows us to specify the service name, the "friendly" name, startup parameters
    /// and many other parameters including the account to run under, and the failure behaviour.
    /// 
    /// Arguably, any service should know how to install and uninstall itself.
    /// 
    /// ms-help://MS.VSCC.2003/MS.MSDNQTR.2004JUL.1033/cpref/html/frlrfsystemserviceprocessserviceinstallerclasstopic.htm
    /// </summary>
    [RunInstaller(true)]
    public class YacServiceInstaller : System.Configuration.Install.Installer
    {
        ServiceProcessInstaller processInstaller;
        ServiceInstaller serviceInstaller;

        string myServiceName;
        string myDisplayName;
        string myDescription;

        public YacServiceInstaller()
        {
            //InitializeComponent();
            Initialize(Resources.ServiceName, Resources.ServiceName, Resources.ServiceDescription);
        }

        public YacServiceInstaller(string serviceName, string displayName, string description)
        {
            Initialize(serviceName, displayName, description);
        }

        /// <summary>
        /// Initialize the installer
        /// </summary>
        /// <param name="serviceName"></param>
        /// <param name="displayName"></param>
        /// <param name="description"></param>
        public void Initialize(string serviceName, string displayName, string description)
        {
            myServiceName = serviceName;
            myDisplayName = displayName;
            myDescription = description;

            // Instantiate installers for process and services.
            processInstaller = new ServiceProcessInstaller();
            serviceInstaller = new ServiceInstaller();

            // The services run under the system account.
            processInstaller.Account = ServiceAccount.LocalSystem;

            // The services are started automatically.
            serviceInstaller.StartType = ServiceStartMode.Automatic;

            // ServiceName must equal those on ServiceBase derived classes.            
            serviceInstaller.ServiceName = myServiceName;

            // displayed in list
            serviceInstaller.DisplayName = myDisplayName;

            // Add installers to collection. Order is not important.
            Installers.Add(serviceInstaller);
            Installers.Add(processInstaller);

        }


        /// <summary>
        /// http://www.codeproject.com/dotnet/dotnetscmdescription.asp
        /// </summary>
        /// <param name="stateServer"></param>
        public override void Install(IDictionary stateSaver)
        {
            Microsoft.Win32.RegistryKey system,
                //HKEY_LOCAL_MACHINE\Services\CurrentControlSet
                currentControlSet,
                //...\Services
                services,
                //...\<Service Name>
                service;//,
                //...\Parameters - this is where you can put service-specific configuration
                //config;

            try
            {
                //Let the project installer do its job
                base.Install(stateSaver);

                //Open the HKEY_LOCAL_MACHINE\SYSTEM key
                system = Microsoft.Win32.Registry.LocalMachine.OpenSubKey("System");
                //Open CurrentControlSet
                currentControlSet = system.OpenSubKey("CurrentControlSet");
                //Go to the services key
                services = currentControlSet.OpenSubKey("Services");
                //Open the key for your service, and allow writing
                service = services.OpenSubKey(serviceInstaller.ServiceName, true);

                //Add your service's description as a REG_SZ value named "Description"
                service.SetValue("Description", myDescription);

                //Add a parameter that will cause the executable to run as a service
                //config = service.CreateSubKey("Parameters");
                //config.SetValue("Arguments", command);
                //Console.WriteLine(service.GetValue("ImagePath"));

                string path = service.GetValue("ImagePath") + " /s";
                service.SetValue("ImagePath", path);
            }
            catch (Exception e)
            {
                Trace.TraceError("An exception was thrown during service installation:\n" + e.ToString());
                Console.WriteLine("An exception was thrown during service installation:\n" + e.ToString());
                throw e;
            }
        }


        /// <summary>
        /// http://www.codeproject.com/dotnet/dotnetscmdescription.asp
        /// </summary>
        /// <param name="stateServer"></param>
        public override void Uninstall(IDictionary savedState)
        {

            // if service was not previously selected for installation,
            // we should not try to uninstall it.
            //if (savedState == null)
            //{
            //    return;
            //}

            Microsoft.Win32.RegistryKey system,
                currentControlSet,
                services,
                service;

            try
            {
                //Drill down to the service key and open it with write permission
                system = Microsoft.Win32.Registry.LocalMachine.OpenSubKey("System");
                currentControlSet = system.OpenSubKey("CurrentControlSet");
                services = currentControlSet.OpenSubKey("Services");
                service = services.OpenSubKey(serviceInstaller.ServiceName, true);

                //Delete any keys you created during installation (or that your service created)
                //service.DeleteSubKeyTree("Parameters");
                //...
            }
            catch (Exception e)
            {
                Console.WriteLine("Exception encountered while uninstalling service:\n" + e.ToString());
                throw e;
            }
            finally
            {
                //Let the project installer do its job
                base.Uninstall(savedState);
            }
        }


        /// <summary> 
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
            }
            base.Dispose(disposing);
        }

        /// <summary>
        /// Install the service
        /// </summary>
        /// <param name="serviceName"></param>
        /// <param name="displayName"></param>
        /// <param name="description"></param>
        public static void DoInstall(string serviceName, string displayName, string description)
        {

            try
            {
                Console.WriteLine("Installing service...");
                TransactedInstaller ti = new TransactedInstaller();
                YacServiceInstaller mi = new YacServiceInstaller(serviceName, displayName, description);
                ti.Installers.Add(mi);
                String path = String.Format("/assemblypath={0}", Assembly.GetExecutingAssembly().Location);
                String[] cmdline = { path };
                InstallContext ctx = new InstallContext("", cmdline);
                ti.Context = ctx;
                ti.Install(new Hashtable());
                Console.WriteLine("Service installed.");
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.ToString());
            }
        }

        /// <summary>
        /// Uninstall the service
        /// </summary>
        /// <param name="serviceName"></param>
        /// <param name="displayName"></param>
        /// <param name="description"></param>
        public static void DoUninstall(string serviceName, string displayName, string description)
        {
            try
            {
                Console.WriteLine("Uninstalling service...");
                TransactedInstaller ti = new TransactedInstaller();
                YacServiceInstaller mi = new YacServiceInstaller(serviceName, displayName, description);
                ti.Installers.Add(mi);
                String path = String.Format("/assemblypath={0}", Assembly.GetExecutingAssembly().Location);
                String[] cmdline = { path };
                InstallContext ctx = new InstallContext("", cmdline);
                ti.Context = ctx;
                ti.Uninstall(null);
                Console.WriteLine("Service uninstalled.");
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.ToString());
            }
        }


        // Prompt the user for service installation account values.
        // ms-help://MS.VSCC.v80/MS.MSDN.v80/MS.NETDEVFX.v20.en/cpref11/html/T_System_ServiceProcess_Design_ServiceInstallerDialog.htm
        //public static bool GetServiceAccount(ref ServiceProcessInstaller svcInst)
        //{
        //    bool accountSet = false;
        //    ServiceInstallerDialog svcDialog = new ServiceInstallerDialog();
        //    // Query the user for the service account type.
        //    do
        //    {
        //        svcDialog.TopMost = true;
        //        svcDialog.ShowDialog();
        //        if (svcDialog.Result == ServiceInstallerDialogResult.OK)
        //        {
        //            // Do a very simple validation on the user
        //            // input.  Check to see whether the user name
        //            // or password is blank.
        //            if ((svcDialog.Username.Length > 0) &&
        //                (svcDialog.Password.Length > 0))
        //            {
        //                // Use the account and password.
        //                accountSet = true;
        //                svcInst.Account = ServiceAccount.User;
        //                svcInst.Username = svcDialog.Username;
        //                svcInst.Password = svcDialog.Password;
        //            }
        //        }
        //        else if (svcDialog.Result == ServiceInstallerDialogResult.UseSystem)
        //        {
        //            svcInst.Account = ServiceAccount.LocalSystem;
        //            svcInst.Username = null;
        //            svcInst.Password = null;
        //            accountSet = true;
        //        }
        //        if (!accountSet)
        //        {
        //            // Display a message box.  Tell the user to
        //            // enter a valid user and password, or cancel
        //            // out to leave the service account alone.
        //            DialogResult result;
        //            result = MessageBox.Show("Invalid user name or password for service installation." +
        //                                     "  Press Cancel to leave the service account unchanged.",
        //                                     "Change Service Account",
        //                                     MessageBoxButtons.OKCancel,
        //                                     MessageBoxIcon.Hand);
        //            if (result == DialogResult.Cancel)
        //            {
        //                // Break out of loop.
        //                break;
        //            }
        //        }
        //    } while (!accountSet);
        //    return accountSet;
        //}


    }
}
