//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//=============================================================================
using System;
using System.Threading;
using System.Diagnostics;
using System.Windows.Forms;

namespace YAC_NET
{
    public enum Scope
    {
        Global,
        Local,
        Session,
        none
    }

    /// <summary>
    /// Summary description for SingleInstance.
    /// Use this global version to ensure a single instance even across
    /// terminal services, Citrix or XP multi-user sessions.
    /// Use a GUID string in place of "MyApp".
    /// instanceLock = new Mutex(false, "Global\MyApp");
    /// Use this local version to ensure a single instance per user session.
    /// </summary>
    public class SingleInstance
    {
        static Mutex instanceLock = null;

        /// <summary>
        /// Create a mutex to ensure that this application only runs one instance.
        /// http://www.ai.uga.edu/mc/SingleInstance.html
        /// http://www.codeproject.com/csharp/cssingprocess.asp
        /// </summary>
        public static bool IsSingleInstance()
        {
            return IsSingleInstance(Scope.none, Application.ProductName);
        }


        private static string GetScope(Scope scope)
        {
            switch (scope)
            {
                case Scope.Global: return @"Global\";
                case Scope.Local: return @"Local\"; 
                case Scope.Session: return @"Session\";
                case Scope.none: return string.Empty;
            }
            return string.Empty;
        }

        public static bool IsSingleInstance(string instanceName)
        {
            return IsSingleInstance(Scope.none, instanceName);
        }

        public static bool IsSingleInstance(Scope scope, string instanceName)
        {
            bool created = false;
            string l_instanceName = string.Format("{0}{1}", GetScope(scope), instanceName);
            instanceLock = new Mutex(true, l_instanceName, out created);
            if (!created)
            {
                Trace.TraceWarning("Program {0} is already running", Application.ProductName);
                return false;
            }

            return true;
        }

    }
}
