//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//=============================================================================
using System;
using System.Windows.Forms;
using System.Diagnostics;

namespace YAC_NET
{
    /// <summary>
    /// Listens to trace events and print them to a textbox
    /// </summary>
    public class TextBoxTraceListener : TraceListener
    {
        static TextBoxBase myTextBox;

        /// <summary>
        /// Constructor, must provide a text box
        /// </summary>
        /// <param name="textBox"></param>
        public TextBoxTraceListener(TextBoxBase textBox)
        {
            myTextBox = textBox;
            Trace.Listeners.Add(this);
        }

        #region singleton pattern
        private TextBoxTraceListener()
        {
            Trace.Listeners.Add(this);
        }
        private static TextBoxTraceListener myInstance;
        public static TextBoxTraceListener P
        {
            get
            {
                if (myInstance == null)
                    myInstance = new TextBoxTraceListener();
                return myInstance;
            }
        }
        #endregion

        /// <summary>
        /// the textbox to persist to
        /// </summary>
        public TextBoxBase TextBox
        {
            get { return myTextBox; }
            set { myTextBox = value; }
        }

        /// <summary>
        /// Write a message to the trace log
        /// overridden from the TraceListener base class
        /// </summary>
        /// <param name="message"></param>
        public override void Write(string message)
        {
            if (myTextBox == null)
                return;

            if (myTextBox.InvokeRequired)
            {
                myTextBox.BeginInvoke( (MethodInvoker)delegate() { Write(message); });
                return;
            }

            myTextBox.AppendText(message);
        }

        /// <summary>
        /// Write a message to the trace log
        /// overridden from the TraceListener base class
        /// </summary>
        /// <param name="message"></param>
        public override void WriteLine(string message)
        {
            if (myTextBox == null)
                return;

            if (myTextBox.InvokeRequired)
            {
                myTextBox.BeginInvoke((MethodInvoker)delegate() { WriteLine(message); });
                return;
            }
            myTextBox.AppendText(message + "\r\n");
        }

    }
}
