//=============================================================================
//
// YAC.NET: Yet another Caller ID Program, .NET Edition
// Copyright  2007 Michael Coyle (michael.coyle@BlueToque.ca)
// http://www.BlueToque.ca/Products/YAC.NET.html
//
//=============================================================================
using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.ServiceProcess;
using System.Diagnostics;

using YAC_NET.Properties;

namespace YAC_NET.Utility
{
    class Utility
    {
        /// <summary>
        /// Enumerate the services looking for one configures with the name in the resource file
        /// If it is installed, then this program has been configured to
        /// run as a service
        /// </summary>
        /// <param name="p"></param>
        /// <returns></returns>
        internal static bool ServiceExists(string p)
        {
            try
            {
                ServiceController[] controllers = ServiceController.GetServices();
                foreach (ServiceController service in controllers)
                {
                    if (service.ServiceName == Resources.ServiceName)
                        return true;
                }
            }
            catch (Exception ex)
            {
                Trace.TraceError("Error enumerating services:\r\n{0}", ex.ToString());
            }

            return false;
        }

        /// <summary>
        /// Formula to select a DatagridView row on right click
        /// Has to be on the MouseDown event of the DataGridView and not the CellMouseDown event!
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        internal static void SelectDataGridViewRowOnRightClick(object sender, MouseEventArgs e)
        {
            // make sure it's a right click
            if (e.Button != MouseButtons.Right)
                return;

            // Get the DatagridView
            DataGridView view = sender as DataGridView;
            if (view == null) return;

            // Get the BindingSource
            BindingSource bindingSource = view.DataSource as BindingSource;
            if (bindingSource == null) return;

            // clear the current selection
            view.ClearSelection();

            // do a hit test and make sire we right clicked on a row
            DataGridView.HitTestInfo hti = view.HitTest(e.X, e.Y);
            if (hti.Type != DataGridViewHitTestType.Cell)
                return;

            // make sure the index selected is within the list members
            if (hti.RowIndex < 0 || hti.RowIndex >= bindingSource.Count)
                return;

            // get the object we clicked on
            object dbi = view.Rows[hti.RowIndex].DataBoundItem;

            // select that object in the BindingSource
            // This will update the DataGridView properly
            bindingSource.Position = bindingSource.IndexOf(dbi);

            // resetting the binding here forces the list to re-select even if the bindingsource's position
            // did not change. If this is not called, and we're just selecting the same item in the list, 
            // since we cleared the selection (above) there will be no new selection and the DataGridView
            // will apear to not have a selection.
            bindingSource.ResetBindings(false);

        }
    }
}
